!-------------------------------------------------------------------------
! f90gl Version 1.0 September 1996
! 
! William F. Mitchell
! Applied and Computational Mathematics Division
! National Institute of Standards and Technology
! Gaithersburg, MD 20899
! william.mitchell@nist.gov
! 
! This fortran interface for GLUT software and documentation have been produced
! as part of work done by the U.S. Government, and are not subject to copyright
! in the United States.
! 
! The mention of specific products, trademarks, or brand names in the
! documentation is for purposes of identification only.  Such mention is not to
! be interpreted in any way as an endorsement or certification of such products
! or brands by the National Institute of Standards and Technology.
! All trademarks mentioned herein belong to their respective owners.
!-------------------------------------------------------------------------

!  blender renders two spinning icosahedrons (red and green).
!  The blending factors for the two icosahedrons vary sinusoidally
!  and slightly out of phase.  blender also renders two lines of
!  text in a stroke font: one line antialiased, the other not.

module blender_data

! with use here, we don't need them in each routine
use f90gl
use f90glu
use f90glut

real(f90glfloat) :: light0_ambient(4) = (/0.2, 0.2, 0.2, 1.0/)
real(f90glfloat) :: light0_diffuse(4) = (/0.0, 0.0, 0.0, 1.0/)
real(f90glfloat) :: light1_diffuse(4) = (/1.0, 0.0, 0.0, 1.0/)
real(f90glfloat) :: light1_position(4) = (/1.0, 1.0, 1.0, 0.0/)
real(f90glfloat) :: light2_diffuse(4) = (/0.0, 1.0, 0.0, 1.0/)
real(f90glfloat) :: light2_position(4) = (/-1.0, -1.0, 1.0, 0.0/)
real :: s = 0.0
real(f90glfloat) :: angle1 = 0.0, angle2 = 0.0

end module blender_data

subroutine output(x, y, text)
use blender_data
real(f90glfloat) x,y
character(len=*) text
integer(f90glcint) p
external glut_stroke_roman

  call f90glPushMatrix()
  call f90glTranslatef(x, y, 0.0_f90glfloat)
  do i=1,len(text)
    p = ichar(text(i:i))
    call f90glutStrokeCharacter(GLUT_STROKE_ROMAN, p)
  end do
  call f90glPopMatrix()
end subroutine output

subroutine display()
use blender_data

  real(f90glfloat), save :: amb(4) = (/0.4, 0.4, 0.4, 0.0/)
  real(f90glfloat), save :: dif(4) = (/1.0, 1.0, 1.0, 0.0/)

  call f90glClear(ior(GL_COLOR_BUFFER_BIT,GL_DEPTH_BUFFER_BIT))
  call f90glEnable(GL_LIGHT1)
  call f90glDisable(GL_LIGHT2)
  dif(4) = cos(s) / 2.0 + 0.5
  amb(4) = dif(4)
  call f90glMaterialfv(GL_FRONT, GL_AMBIENT, amb)
  call f90glMaterialfv(GL_FRONT, GL_DIFFUSE, dif)

  call f90glPushMatrix()
! let's take a chance that the default integer is the same kind as
! f90glint, and not bother with the _f90glint on constants
  call f90glTranslatef(-0.3, -0.3, 0.0)
  call f90glRotatef(angle1, 1.0, 5.0, 0.0)
  call f90glCallList(1)        ! render ico display list
  call f90glPopMatrix()

  call f90glClear(GL_DEPTH_BUFFER_BIT)
  call f90glEnable(GL_LIGHT2)
  call f90glDisable(GL_LIGHT1)
  dif(4) = 0.5 - cos(s * .95) / 2.0
  amb(4) = dif(4)
  call f90glMaterialfv(GL_FRONT, GL_AMBIENT, amb)
  call f90glMaterialfv(GL_FRONT, GL_DIFFUSE, dif)

  call f90glPushMatrix()
  call f90glTranslatef(0.3, 0.3, 0.0)
  call f90glRotatef(angle2, 1.0, 0.0, 5.0)
  call f90glCallList(1)        ! render ico display list
  call f90glPopMatrix()

! Don't do the text.  The text caused problems when I upgraded to
! Mesa 1.2.8; It was be a bug in Mesa 1.2.8, fixed in Mesa 2.0
!  call f90glPushAttrib(GL_ENABLE_BIT)
!  call f90glDisable(GL_DEPTH_TEST)
!  call f90glDisable(GL_LIGHTING)
!  call f90glMatrixMode(GL_PROJECTION)
!  call f90glPushMatrix()
!  call f90glLoadIdentity()
!  call f90gluOrtho2D(0.0_f90gldouble, 1500.0_f90gldouble, 0.0_f90gldouble, 1500.0_f90gldouble)
!  call f90glMatrixMode(GL_MODELVIEW)
!  call f90glPushMatrix()
!  call f90glLoadIdentity()
!  Rotate text slightly to help show jaggies.
!  call f90glRotatef(4.0, 0.0, 0.0, 1.0)
!  call output(200., 225., "This is antialiased.")
!  call f90glDisable(GL_LINE_SMOOTH)
!  call f90glDisable(GL_BLEND)
!  call output(160., 100., "This text is not.")
!  call f90glPopMatrix()
!  call f90glMatrixMode(GL_PROJECTION)
!  call f90glPopMatrix()
!  call f90glPopAttrib()
!  call f90glMatrixMode(GL_MODELVIEW)

  call f90glutSwapBuffers()
end subroutine display

subroutine idle()
use blender_data

  angle1 = mod(angle1 + 0.8, 360.0)
  angle2 = mod(angle2 + 1.1, 360.0)
  s = s + 0.05
  call f90glutPostRedisplay()
end subroutine idle

subroutine visible(vis)
use blender_data
integer vis
external idle
external glutnull

  if (vis == GLUT_VISIBLE) then
    call f90glutIdleFunc(idle)
  else
    call f90glutIdleFunc(glutnull)
  endif
end subroutine visible

program main
use blender_data
use f90glut
integer(f90glcint) i
external display
external visible

  call f90glutInit()
  call f90glutInitDisplayMode(ior(ior(GLUT_DOUBLE,GLUT_RGB),GLUT_DEPTH))
  i = f90glutCreateWindow("blender")
  call f90glutDisplayFunc(display)
  call f90glutVisibilityFunc(visible)

  call f90glNewList(1, GL_COMPILE)  ! create ico display list
  call f90glutSolidIcosahedron()
  call f90glEndList()

  call f90glEnable(GL_LIGHTING)
  call f90glEnable(GL_LIGHT0)
  call f90glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient)
  call f90glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse)
  call f90glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse)
  call f90glLightfv(GL_LIGHT1, GL_POSITION, light1_position)
  call f90glLightfv(GL_LIGHT2, GL_DIFFUSE, light2_diffuse)
  call f90glLightfv(GL_LIGHT2, GL_POSITION, light2_position)
  call f90glEnable(GL_DEPTH_TEST)
  call f90glEnable(GL_CULL_FACE)
  call f90glEnable(GL_BLEND)
  call f90glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
  call f90glEnable(GL_LINE_SMOOTH)
  call f90glLineWidth(2.0)

  call f90glMatrixMode(GL_PROJECTION)
  call f90gluPerspective( 40.0_f90gldouble, & ! field of view in degree
                           1.0_f90gldouble, & ! aspect ratio
                           1.0_f90gldouble, & ! Z near
                          10.0_f90gldouble)   ! Z far
  call f90glMatrixMode(GL_MODELVIEW)
  call f90gluLookAt( &
     0.0_f90gldouble, 0.0_f90gldouble, 5.0_f90gldouble, & ! eye is at (0,0,5)
     0.0_f90gldouble, 0.0_f90gldouble, 0.0_f90gldouble, & ! center is at (0,0,0)
     0.0_f90gldouble, 1.0_f90gldouble, 0.0_f90gldouble)    ! up is in positive Y direction
  call f90glTranslatef(0.0, 0.6, -1.0)

  call f90glutMainLoop()
end program main
