!-------------------------------------------------------------------------
! f90gl Version 1.0 September 1996
! 
! William F. Mitchell
! Applied and Computational Mathematics Division
! National Institute of Standards and Technology
! Gaithersburg, MD 20899
! william.mitchell@nist.gov
! 
! This fortran interface for GLUT software and documentation have been produced
! as part of work done by the U.S. Government, and are not subject to copyright
! in the United States.
! 
! The mention of specific products, trademarks, or brand names in the
! documentation is for purposes of identification only.  Such mention is not to
! be interpreted in any way as an endorsement or certification of such products
! or brands by the National Institute of Standards and Technology.
! All trademarks mentioned herein belong to their respective owners.
!-------------------------------------------------------------------------

module scube_mod
use f90gl
use f90glut
implicit none

logical, save :: &
   useRGB      = .true., &
   useLighting = .true., &
   useFog      = .false., &
   useDB       = .true., &
   useLogo     = .false., &
   useQuads    = .true.

integer, save :: tick = -1
logical, save :: moving = .true.

integer, parameter :: &
   GREY    = 0, &
   RED     = 1, &
   GREEN   = 2, &
   BLUE    = 3, &
   CYAN    = 4, &
   MAGENTA = 5, &
   YELLOW  = 6, &
   BLACK   = 7

real(f90glfloat), save :: materialColor(8,4) = reshape(reshape( &
  (/ 0.8, 0.8, 0.8, 1.0, &
     0.8, 0.0, 0.0, 1.0, &
     0.0, 0.8, 0.0, 1.0, &
     0.0, 0.0, 0.8, 1.0, &
     0.0, 0.8, 0.8, 1.0, &
     0.8, 0.0, 0.8, 1.0, &
     0.8, 0.8, 0.0, 1.0, &
     0.0, 0.0, 0.0, 0.6/), &
  (/4,8/)),(/8,4/),order=(/2,1/))

real(f90glfloat), save :: &
 lightPos(4) = (/2.0, 4.0, 2.0, 1.0/), &
 lightDir(4) = (/-2.0, -4.0, -2.0, 1.0/), &
 lightAmb(4) = (/0.2, 0.2, 0.2, 1.0/), &
 lightDiff(4) = (/0.8, 0.8, 0.8, 1.0/), &
 lightSpec(4) = (/0.4, 0.4, 0.4, 1.0/)

real(f90glfloat), save :: &
 groundPlane(4) = (/0.0, 1.0, 0.0, 1.499/), &
 backPlane(4) = (/0.0, 0.0, 1.0, 0.899/)

real(f90glfloat), save :: &
 fogColor(4) = (/0.0, 0.0, 0.0, 0.0/), &
 fogIndex(1) = (/0.0/)

integer(f90glubyte), save :: shadowPattern(128) ! 50% Grey
data shadowPattern / &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55', &
  z'aa', z'aa', z'aa', z'aa', z'55', z'55', z'55', z'55' /

integer(f90glubyte), save :: sgiPattern(128) ! SGI Logo
data sgiPattern / &
  z'ff', z'ff', z'ff', z'ff', z'ff', z'ff', z'ff', z'ff', &
  z'ff', z'bd', z'ff', z'83', z'ff', z'5a', z'ff', z'ef', &
  z'fe', z'db', z'7f', z'ef', z'fd', z'db', z'bf', z'ef', &
  z'fb', z'db', z'df', z'ef', z'f7', z'db', z'ef', z'ef', &
  z'fb', z'db', z'df', z'ef', z'fd', z'db', z'bf', z'83', &
  z'ce', z'db', z'73', z'ff', z'b7', z'5a', z'ed', z'ff', &
  z'bb', z'db', z'dd', z'c7', z'bd', z'db', z'bd', z'bb', &
  z'be', z'bd', z'7d', z'bb', z'bf', z'7e', z'fd', z'b3', &
  z'be', z'e7', z'7d', z'bf', z'bd', z'db', z'bd', z'bf', &
  z'bb', z'bd', z'dd', z'bb', z'b7', z'7e', z'ed', z'c7', &
  z'ce', z'db', z'73', z'ff', z'fd', z'db', z'bf', z'ff', &
  z'fb', z'db', z'df', z'87', z'f7', z'db', z'ef', z'fb', &
  z'f7', z'db', z'ef', z'fb', z'fb', z'db', z'df', z'fb', &
  z'fd', z'db', z'bf', z'c7', z'fe', z'db', z'7f', z'bf', &
  z'ff', z'5a', z'ff', z'bf', z'ff', z'bd', z'ff', z'c3', &
  z'ff', z'ff', z'ff', z'ff', z'ff', z'ff', z'ff', z'ff' /


character(len=30), save :: windowNameRGBDB = "shadow cube (OpenGL RGB DB)"
character(len=30), save :: windowNameRGB = "shadow cube (OpenGL RGB)"
character(len=30), save :: windowNameIndexDB = "shadow cube (OpenGL Index DB)"
character(len=30), save :: windowNameIndex = "shadow cube (OpenGL Index)"

end module scube_mod

subroutine buildColormap()
use scube_mod

integer mapSize,rampSize,entry,i,hue
real(f90glfloat) val,r,g,b

mapSize = 2**f90glutGet(GLUT_WINDOW_BUFFER_SIZE)
rampSize = mapSize / 8

  if (useRGB) then
    return
  else

    do entry=0,mapSize-1
      hue = entry / rampSize
      val = mod(entry,rampSize) * (1.0 / (rampSize - 1))

      if (hue==0 .or. hue==1 .or. hue==5 .or. hue==6) then
         r = val
      else
         r = 0
      endif
      if (hue==0 .or. hue==2 .or. hue==4 .or. hue==6) then
         g = val
      else
         g = 0
      endif
      if (hue==0 .or. hue==3 .or. hue==4 .or. hue==5) then
         b = val
      else
         b = 0
      endif

      call f90glutSetColor(entry, r, g, b);
    end do

    do i=1,8
      materialColor(i,1) = i * rampSize + 0.2 * (rampSize - 1)
      materialColor(i,2) = i * rampSize + 0.8 * (rampSize - 1)
      materialColor(i,3) = i * rampSize + 1.0 * (rampSize - 1)
      materialColor(i,4) = 0.0
    end do

    fogIndex(1) = -0.2 * (rampSize - 1)
  endif
end subroutine buildColormap

subroutine setColor(c)
use scube_mod
integer c
! had to move materialColor to here because of bug in SGI f90 compiler
real(f90glfloat), save :: materialCol(8,4) = reshape(reshape( &
  (/ 0.8, 0.8, 0.8, 1.0, &
     0.8, 0.0, 0.0, 1.0, &
     0.0, 0.8, 0.0, 1.0, &
     0.0, 0.0, 0.8, 1.0, &
     0.0, 0.8, 0.8, 1.0, &
     0.8, 0.0, 0.8, 1.0, &
     0.8, 0.8, 0.0, 1.0, &
     0.0, 0.0, 0.0, 0.6/), &
  (/4,8/)),(/8,4/),order=(/2,1/))


  if (useLighting) then
    if (useRGB) then
      call f90glMaterialfv(GL_FRONT_AND_BACK, &
        GL_AMBIENT_AND_DIFFUSE, materialCol(c+1,:))
    else
      call f90glMaterialfv(GL_FRONT_AND_BACK, &
        GL_COLOR_INDEXES, materialColor(c+1,:))
    endif
  else
    if (useRGB) then
      call f90glColor4fv(materialCol(c+1,:))
    else
      call f90glIndexf(materialColor(c+1,1))
    endif
  endif
end subroutine setColor

subroutine drawCube(color)
use scube_mod
integer color

real(f90glfloat), save :: cube_vertexes(4,4,6) = reshape( (/ &
    -1.0, -1.0, -1.0, 1.0, &
    -1.0, -1.0, 1.0, 1.0, &
    -1.0, 1.0, 1.0, 1.0, &
    -1.0, 1.0, -1.0, 1.0, &

    1.0, 1.0, 1.0, 1.0, &
    1.0, -1.0, 1.0, 1.0, &
    1.0, -1.0, -1.0, 1.0, &
    1.0, 1.0, -1.0, 1.0, &

    -1.0, -1.0, -1.0, 1.0, &
    1.0, -1.0, -1.0, 1.0, &
    1.0, -1.0, 1.0, 1.0, &
    -1.0, -1.0, 1.0, 1.0, &

    1.0, 1.0, 1.0, 1.0, &
    1.0, 1.0, -1.0, 1.0, &
    -1.0, 1.0, -1.0, 1.0, &
    -1.0, 1.0, 1.0, 1.0, &

    -1.0, -1.0, -1.0, 1.0, &
    -1.0, 1.0, -1.0, 1.0, &
    1.0, 1.0, -1.0, 1.0, &
    1.0, -1.0, -1.0, 1.0, &

    1.0, 1.0, 1.0, 1.0, &
    -1.0, 1.0, 1.0, 1.0, &
    -1.0, -1.0, 1.0, 1.0, &
    1.0, -1.0, 1.0, 1.0 /), &
    (/4,4,6/) )

real(f90glfloat), save :: cube_normals(4,6) = reshape( (/ &
  -1.0, 0.0, 0.0, 0.0, &
  1.0, 0.0, 0.0, 0.0, &
  0.0, -1.0, 0.0, 0.0, &
  0.0, 1.0, 0.0, 0.0, &
  0.0, 0.0, -1.0, 0.0, &
  0.0, 0.0, 1.0, 0.0 /), &
  (/4,6/) )

  integer i

  call setColor(color)

  do i=1,6
    call f90glNormal3fv(cube_normals(:,i))
    call f90glBegin(GL_POLYGON)
    call f90glVertex4fv(cube_vertexes(:,1,i))
    call f90glVertex4fv(cube_vertexes(:,2,i))
    call f90glVertex4fv(cube_vertexes(:,3,i))
    call f90glVertex4fv(cube_vertexes(:,4,i))
    call f90glEnd()
  end do
end subroutine drawCube

subroutine drawCheck(w,h,evenColor,oddColor)
use scube_mod
integer w,h,evenColor,oddColor

  logical, save :: initialized = .false., &
                   usedLighting = .false.
  integer(f90gluint), save :: checklist = 0
  real, save :: square_normal(4) = (/0.0, 0.0, 1.0, 0.0/)
  real, save :: square(4,4)
  integer i,j

  if (.not. initialized .or. (usedLighting .EQV. useLighting)) then

    if (checklist == 0) then
      checklist = f90glGenLists(1)
    endif
    call f90glNewList(checklist, GL_COMPILE_AND_EXECUTE)

    if (useQuads) then
      call f90glNormal3fv(square_normal)
      call f90glBegin(GL_QUADS)
    endif
    do j=0,h-1
      do i=0,w-1
        square(:,1) = (/ -1.0 + 2.0/w * i, -1.0 + 2.0/h * (j+1), 0.0, 1.0/)
        square(:,2) = (/ -1.0 + 2.0/w * i, -1.0 + 2.0/h * j, 0.0, 1.0/)
        square(:,3) = (/ -1.0 + 2.0/w * (i+1), -1.0 + 2.0/h * j, 0.0, 1.0/)
        square(:,4) = (/ -1.0 + 2.0/w * (i+1), -1.0 + 2.0/h * (j+1), 0.0, 1.0/)

        if (ieor(iand(i,1),iand(j,1)) /= 0) then
          call setColor(oddColor)
        else
          call setColor(evenColor)
        endif

        if (.not.useQuads) then
          call f90glBegin(GL_POLYGON)
        endif
        call f90glVertex4fv(square(:,1))
        call f90glVertex4fv(square(:,2))
        call f90glVertex4fv(square(:,3))
        call f90glVertex4fv(square(:,4))
        if (.not.useQuads) then
          call f90glEnd()
        endif
      end do
    end do

    if (useQuads) then
      call f90glEnd()
    endif
    call f90glEndList()

    initialized = .true.
    usedLighting = useLighting
  else
    call f90glCallList(checklist)
  endif
end subroutine drawCheck

subroutine myShadowMatrix(ground,light)
use scube_mod
real ground(4), light(4)

  real dot
  real(f90glfloat) shadowMat(4,4)
  integer i

  dot = dot_product(ground,light)

  do i=1,4
     shadowMat(i,:) = -light(i)*ground
     shadowMat(i,i) = shadowMat(i,i) + dot
  end do

  call f90glMultMatrixf(shadowMat)
end subroutine myShadowMatrix

subroutine idle()
use scube_mod

  tick = tick + 1
  if (tick >= 120) then
    tick = 0
  endif
  call f90glutPostRedisplay()
end subroutine idle

subroutine keyboard(ich, x, y)
use scube_mod
integer ich,x,y

  character ch
  real(f90glfloat) rGL_LINEAR, rGL_EXP, rGL_EXP2

  ch = achar(ich)
  select case(ch)
  case (achar(27))      ! escape
    stop
  case ('L','l')
    useLighting = .not. useLighting
    if (useLighting) then
       call f90glEnable(GL_LIGHTING) 
    else
       call f90glDisable(GL_LIGHTING)
    endif
    call f90glutPostRedisplay()
  case ('F','f')
    useFog = .not. useFog
    if (useFog) then
       call f90glEnable(GL_FOG)
    else
       call f90glDisable(GL_FOG)
    endif
    call f90glutPostRedisplay()
  case ('1')
    rGL_LINEAR = GL_LINEAR
    call f90glFogf(GL_FOG_MODE, rGL_LINEAR)
    call f90glutPostRedisplay()
  case ('2')
    rGL_EXP = GL_EXP
    call f90glFogf(GL_FOG_MODE, rGL_EXP)
    call f90glutPostRedisplay()
  case ('3')
    rGL_EXP2 = GL_EXP2
    call f90glFogf(GL_FOG_MODE, rGL_EXP2)
    call f90glutPostRedisplay()
  case (' ')
    if (.not. moving) then
      call idle()
      call f90glutPostRedisplay()
    endif
  end select
end subroutine keyboard

subroutine display()
use scube_mod

  real(f90glfloat) cubeXform(16)

  call f90glClear(ior(GL_COLOR_BUFFER_BIT,GL_DEPTH_BUFFER_BIT))

  call f90glPushMatrix()
  call f90glTranslatef(0.0, -1.5, 0.0) ! taking a chance that f90glfloat is
  call f90glRotatef(-90.0, 1., 0., 0.)    ! the same as the default real
  call f90glScalef(2.0, 2.0, 2.0)

  call drawCheck(6, 6, BLUE, YELLOW)  ! draw ground
  call f90glPopMatrix()

  call f90glPushMatrix()
  call f90glTranslatef(0.0, 0.0, -0.9)
  call f90glScalef(2.0, 2.0, 2.0)

  call drawCheck(6, 6, BLUE, YELLOW)  ! draw back
  call f90glPopMatrix()

  call f90glPushMatrix()
  call f90glTranslatef(0.0, 0.2, 0.0)
  call f90glScalef(0.3, 0.3, 0.3)
  call f90glRotatef((360.0 / (30 * 1)) * tick, 1., 0., 0.)
  call f90glRotatef((360.0 / (30 * 2)) * tick, 0., 1., 0.)
  call f90glRotatef((360.0 / (30 * 4)) * tick, 0., 0., 1.)
  call f90glScalef(1.0, 2.0, 1.0)
  call f90glGetFloatv(GL_MODELVIEW_MATRIX, cubeXform)

  call drawCube(RED)        ! draw cube
  call f90glPopMatrix()

  call f90glDepthMask(.false._f90glboolean)
  if (useRGB) then
    call f90glEnable(GL_BLEND)
  else
    call f90glEnable(GL_POLYGON_STIPPLE)
  endif
  if (useFog) then
    call f90glDisable(GL_FOG)
  endif
  call f90glPushMatrix()
  call myShadowMatrix(groundPlane, lightPos)
  call f90glTranslatef(0.0, 0.0, 2.0)
  call f90glMultMatrixf(reshape(cubeXform,(/4,4/)))

  call drawCube(BLACK)      ! draw ground shadow
  call f90glPopMatrix()

  call f90glPushMatrix()
  call myShadowMatrix(backPlane, lightPos)
  call f90glTranslatef(0.0, 0.0, 2.0)
  call f90glMultMatrixf(reshape(cubeXform,(/4,4/)))

  call drawCube(BLACK)      ! draw back shadow
  call f90glPopMatrix()

  call f90glDepthMask(.true._f90glboolean)
  if (useRGB) then
    call f90glDisable(GL_BLEND)
  else
    call f90glDisable(GL_POLYGON_STIPPLE)
  endif
  if (useFog) then
    call f90glEnable(GL_FOG)
  endif
  if(useDB) then
    call f90glutSwapBuffers()
  else
    call f90glFlush()
  endif
end subroutine display

subroutine fog_select(fog)
use scube_mod
integer fog

  real(f90glfloat) rfog
  rfog = fog
  call f90glFogf(GL_FOG_MODE, rfog)
  call f90glutPostRedisplay()
end subroutine fog_select

subroutine menu_select(mode)
use scube_mod
integer mode

  external glutnull, idle

  select case (mode)
  case (1)
    moving = .true.
    call f90glutIdleFunc(idle)
  case (2)
    moving = .false.
    call f90glutIdleFunc(glutnull)
  case (3)
    useFog = .not. useFog
    if (useFog) then
       call f90glEnable(GL_FOG)
    else
       call f90glDisable(GL_FOG)
    endif
    call f90glutPostRedisplay()
  case (4)
    useLighting = .not. useLighting
    if (useLighting) then
       call f90glEnable(GL_LIGHTING)
    else
       call f90glDisable(GL_LIGHTING)
    endif
    call f90glutPostRedisplay()
  case (5)
    stop
  end select
end subroutine menu_select

subroutine visible(state)
use scube_mod
integer state

  external glutnull, idle

  if (state == GLUT_VISIBLE) then
    if (moving) then
      call f90glutIdleFunc(idle)
    endif
  else
    if (moving) then
      call f90glutIdleFunc(glutnull)
    endif
  endif
end subroutine visible

program main
use scube_mod
implicit none
external keyboard, display, visible, fog_select, menu_select

  integer :: width = 350, height = 350
  integer i, win
  character(len=30) name
  integer fog_menu
  real(f90glfloat) rGL_EXP

  call f90glutInitWindowSize(width, height)
  call f90glutInit

  ! choose visual
  if (useRGB) then
    if (useDB) then
      call f90glutInitDisplayMode(ior(ior(GLUT_DOUBLE,GLUT_RGB),GLUT_DEPTH))
      name = windowNameRGBDB
    else
      call f90glutInitDisplayMode(ior(ior(GLUT_SINGLE,GLUT_RGB),GLUT_DEPTH))
      name = windowNameRGB
    endif
  else
    if (useDB) then
      call f90glutInitDisplayMode(ior(ior(GLUT_DOUBLE,GLUT_INDEX),GLUT_DEPTH))
      name = windowNameIndexDB
    else
      call f90glutInitDisplayMode(ior(ior(GLUT_SINGLE,GLUT_INDEX),GLUT_DEPTH))
      name = windowNameIndex
    endif
  endif

  win = f90glutCreateWindow(name)

  call buildColormap()

  call f90glutKeyboardFunc(keyboard)
  call f90glutDisplayFunc(display)
  call f90glutVisibilityFunc(visible)

  fog_menu = f90glutCreateMenu(fog_select)
  call f90glutAddMenuEntry("Linear fog", GL_LINEAR)
  call f90glutAddMenuEntry("Exp fog", GL_EXP)
  call f90glutAddMenuEntry("Exp^2 fog", GL_EXP2)

  i = f90glutCreateMenu(menu_select)
  call f90glutAddMenuEntry("Start motion", 1)
  call f90glutAddMenuEntry("Stop motion", 2)
  call f90glutAddMenuEntry("Toggle fog", 3)
  call f90glutAddMenuEntry("Toggle lighting", 4)
  call f90glutAddSubMenu("Fog type", fog_menu)
  call f90glutAddMenuEntry("Quit", 5)
  call f90glutAttachMenu(GLUT_RIGHT_BUTTON)

  ! setup context
  call f90glMatrixMode(GL_PROJECTION)
  call f90glLoadIdentity()
  call f90glFrustum(-1.0_f90gldouble, 1.0_f90gldouble, -1.0_f90gldouble, &
                     1.0_f90gldouble, 1.0_f90gldouble, 3.0_f90gldouble)

  call f90glMatrixMode(GL_MODELVIEW)
  call f90glLoadIdentity()
  call f90glTranslatef(0.0, 0.0, -2.0)

  call f90glEnable(GL_DEPTH_TEST)

  if (useLighting) then
    call f90glEnable(GL_LIGHTING)
  endif
  call f90glEnable(GL_LIGHT0)
  call f90glLightfv(GL_LIGHT0, GL_POSITION, lightPos)
  call f90glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmb)
  call f90glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiff)
  call f90glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpec)
  
   ! call f90glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, lightDir);
   ! call f90glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 80);
   ! call f90glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 25);

  call f90glEnable(GL_NORMALIZE)

  if (useFog) then
    call f90glEnable(GL_FOG)
  endif
  call f90glFogfv(GL_FOG_COLOR, fogColor)
  call f90glFogfv(GL_FOG_INDEX, fogIndex)
  rGL_EXP = GL_EXP
  call f90glFogf(GL_FOG_MODE, rGL_EXP)
  call f90glFogf(GL_FOG_DENSITY, 0.5)
  call f90glFogf(GL_FOG_START, 1.0)
  call f90glFogf(GL_FOG_END, 3.0)

  call f90glEnable(GL_CULL_FACE)
  call f90glCullFace(GL_BACK)

  call f90glShadeModel(GL_SMOOTH)

  call f90glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
  if (useLogo) then
    call f90glPolygonStipple(sgiPattern)
  else
    call f90glPolygonStipple(shadowPattern)
  endif

  call f90glClearColor(0.0, 0.0, 0.0, 1.0)
  call f90glClearIndex(0.)
  call f90glClearDepth(1._f90gldouble)

  call f90glutMainLoop()
end program main
